<?php

namespace Quivers\Bootstrap;

use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Tools\SchemaTool;
use Quivers\Models\Settings\QuiversSetting;
use Quivers\Models\Settings\QuiversTaxSettings;
use Quivers\Models\Settings\StripeSettings;

class Database
{
    /**
     * @var EntityManager
     */
    private $entityManager;

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
        $this->schemaTool = new SchemaTool($this->entityManager);
    }

    /**
     * Installs all registered ORM classes
     */
    public function install()
    {
        $this->schemaTool->updateSchema(
            $this->getClassesMetaData(),
            true 
        );
    }

    /**
     * Drops all registered ORM classes
     */
    public function uninstall()
    {
        $this->schemaTool->dropSchema(
            $this->getClassesMetaData()
        );
    }

    /**
     * @return array
     */
    private function getClassesMetaData()
    {
        return [
            $this->entityManager->getClassMetadata(QuiversSetting::class),
            $this->entityManager->getClassMetadata(QuiversTaxSettings::class),
            $this->entityManager->getClassMetadata(StripeSettings::class)
        ];
    }
}
