<?php

namespace Quivers\Components;

use Shopware\Components\Model\ModelManager;


class SettingService 
{
    /**
     * @var ModelManager
     */
    private $modelManager;

    /**
     * @var Connection
     */
    private $dbalConnection;


    public function __construct(
        ModelManager $modelManager
    ) {

        $this->modelManager = $modelManager;
        $this->dbalConnection = $modelManager->getConnection();

    }

    /**
     * {@inheritdoc}
     *
     * @throws \RuntimeException
     */
    public function get($column, $settingsType = 'general', $currency = 'USD')
    {
        $table = $this->getTableByType($settingsType);
        if($settingsType == 'tax_settings'){
            $tax_settings = $this->dbalConnection->createQueryBuilder()
                ->select($column)
                ->from($table)
                ->where('currency = :currency')
                ->setParameter('currency', $currency)
                ->execute()->fetchColumn();
            return $tax_settings;
        }
        $settings =  $this->dbalConnection->createQueryBuilder()
            ->select($column)
            ->from($table)
            ->execute()->fetchColumn();
        return $settings;
    }
    /**
     * A helper function that returns the proper table name by the given settings type.
     *
     * @param string $settingsType
     *
     * @throws \RuntimeException
     *
     * @return string
     *
     * @see SettingsTable
     */
    private function getTableByType($settingsType)
    {
        switch ($settingsType) {
            case 'general':
                return 'quivers_settings';
            case 'tax_settings':
                return 'quivers_tax_settings';
            case 'stripe_settings':
                return 'quivers_stripe_settings';
            default:
                throw new \RuntimeException('The provided table ' . $settingsType . ' is not supported');
                break;
        }
    }
}
