<?php

use Quivers\Components\QuiversStripe;
use Shopware\Models\Order\Order;

class Shopware_Controllers_Frontend_StripePayment extends Shopware_Controllers_Frontend_Payment
{

    public function indexAction()
    { 
        if(!$this->is_available()){
            $redirectData = [];
            $redirectData['error_name'] = "Minimium order total";
            $redirectData['error_message']  ="The Payment Method is not available for use as it is not fully configured yet";
            $this->redirectToShippingPayment($redirectData);
            return;
        }
        switch ($this->getPaymentShortName()) {
            case 'Quivers Stripe':
                return $this->redirect(['action' => 'authorize', 'forceSecure' => true]);
            default:
                return $this->redirect(['controller' => 'checkout']);
        }
    }



    public function authorizeAction()
    {
        $service = $this->container->get('quivers_plugin.quivers_stripe');
        if (!$service){
            $redirectData = [];
            $redirectData['error_name'] = "Stripe Payment Error";
            $redirectData['error_message']  ="Unable to process payment. Please try again. If the issue persists please contact Quivers at 805-250-4555";
            $this->redirectToShippingPayment($redirectData);
            return;    
        }
        $amount = $this->getAmount();
        $currency = $this->getCurrencyShortName();
        $data = ["amount"=>$amount, "currency"=>$currency];
        $response = $service->createPaymentResponse($this->getUser(), $data);
        if($response["error_code"]){
            $this->redirectToShippingPayment($response["result"]);
            return;
        }
        $charge = $response["result"];
        $order = $this->saveOrderWithCharge($charge);
        $this->finishCheckout($order);

    }

     protected function saveOrderWithCharge(Stripe\Charge $charge)
    {
        $orderNumber = $this->saveOrder(
            $charge->id, 
            $charge->source->id, 
            0
        );
        if (!$orderNumber) {
            return null;
        }
        // Update the cleared date
        $order = $this->get('models')->getRepository('Shopware\\Models\\Order\\Order')->findOneBy([
            'number' => $orderNumber,
        ]);
        $order->setClearedDate(new \DateTime());
        $this->get('models')->flush($order);

        try {
            // Save the order number in the charge description
            $charge->description .= ' / Order ' . $orderNumber;
            $charge->save();
        } catch (Exception $e) {
        }

        return $order;
    }

    protected function finishCheckout(Order $order)
    {
        Shopware()->Container()->get('session')->Quiversstripe = new \ArrayObject([], \ArrayObject::STD_PROP_LIST);
        $this->redirect([
            'controller' => 'checkout',
            'action' => 'finish'
        ]);
    }

    public function is_available() {
        $basket =  $this->getBasket();
        $settingService = $this->container->get('quivers_plugin.setting_service');
        if (!$settingService){
            return false;    
        }
        $minAmount = $settingService->get('min_order_total', 'stripe_settings');
        $amount = (float)$minAmount;
        if ( $basket['AmountNetNumeric']  < $amount ) {
            return false;
        }
        return true;
    }

    public function redirectToShippingPayment($data){
        $redirectData = [
            'controller' => 'checkout',
            'action' => 'shippingPayment',
            'error_code' => 1,
        ];
        $redirectData['error_name'] = $data['error_name'];
        $redirectData['error_message']  = $data['error_message'];

        $this->redirect($redirectData);

    }
}